// Written by Christopher E. Miller
// This code "listener" belongs to the public domain!


import std.conv;
import socket;


int main(char[][] args)
{
	ushort port;
	if(args.length >= 2)
	{
		port = std.conv.toUshort(args[1]);
	}
	else
	{
		port = 4444;
	}
	
	Socket listener = new Socket(AddressFamily.INET, SocketType.STREAM, ProtocolType.IP);
	listener.blocking = false;
	listener.bind(new InternetAddress(port));
	listener.listen(10);
	printf("Listening on port %d.\r\n", cast(int)port);
	
	const int READ_MAX = 60;
	SocketSet sset = new SocketSet(READ_MAX + 1); //room for listener
	Socket[] reads;
	
	for(;; sset.reset())
	{
		sset.add(listener);
		foreach(Socket each; reads)
		{
			sset.add(each);
		}
		
		Socket.select(sset, null, null);
		
		int i;
		for(i = 0;; i++)
		{
			next:
			if(i == reads.length)
				break;

			if(sset.isSet(reads[i]))
			{
				char[1024] buf;
				int read;
				try
				{
					read = reads[i].receive(buf);
				}
				catch(SocketException se)
				{
					se.print();
					goto sock_down;
				}
				if(read > 0)
				{
					printf("Received %d bytes from %.*s: \"%.*s\"\r\n", read, reads[i].remoteAddress().toString(), buf[0 .. read]);
				}
				else
				{
					try
					{
						//if the connection closed due to an error, remoteAddress() could fail
						printf("Connection from %.*s closed.\r\n", reads[i].remoteAddress().toString());
					}
					catch
					{
					}
					
					sock_down:
					reads[i].close(); //release socket resources now

					//remove from -reads-
					if(i != reads.length - 1)
						reads[i] = reads[reads.length - 1];
					reads = reads[0 .. reads.length - 1];

					goto next; //-i- is still the next index
				}
			}
		}
		
		if(sset.isSet(listener)) //connection request
		{
			Socket sn;
			if(reads.length < READ_MAX)
			{
				sn = listener.accept();
				reads ~= sn;
				printf("Connection from %.*s established.\r\n", sn.remoteAddress().toString());
			}
			else
			{
				sn = listener.accept();
				printf("Rejected connection from %.*s; too many connections.\r\n", sn.remoteAddress().toString());
				sn.close();
			}
		}
	}
	
	return 0;
}

